Elasticsearch Schema Utilities
===============================

`build_es_schema.py`
--------------------
This utility validates the elasticsearch schema json and produces the build artifacts for provisioning.  

This script has been integrated into the provision_es.sh script.  The build itself takes place in docker so that no 
additional local dependencies are required.  This build step has been added to build_docker.sh and will fail the build 
if an invalid mappings/settings file is added. 

For running outside of docker, python dependencies can be installed using `pip install -r requirements`

### Usage:
`python build_es_schema.py [output dir]`

* **output dir** Custom directory for build artifact output.  By default artifacts are added to 
`backend/backendImpl/target/elasticsearch`.  Contents of this directory will be removed as a part of the build.

### Build Operation

* During the python build:
  * Mappings and settings json files are validated and as part of the build output they are properly formatted
  * Existing upgrade scripts are copied to the output directory
  * `elasticsearch-dao.xml` is parsed to determine default index names and whether a mapping is for a template or an index
  * A settings file, `default_configuration.yml` is created based on the default sizings, names, mapping/setting files, etc.
  * These artifacts are created in the output directory
* After the build:
  * The `provision_elasticsearch.py` script is available
  * It runs with the `default_configuration.yml` file but any setting can be overridden by specifying a replacement yaml file which will be merged with the defaults
     * specify `--config <environment>.yml` to provide a file with custom shard/replica/index name/es url/etc values
  * Existing indexes/aliases/templates will be removed and the new schema will be applied based on the built mappings

`provision_elasticsearch.py`
----------------------------

This is the script that gets packaged as part of a deployment.  It can provision a fresh instance of elasticsearch.  

**Any existing indexes, templates, and aliases will be removed before the new schema is applied!**

### Usage:
`python provision_elasticsearch.py [--config file.yml] [--no-confirm] [--es-url-override http://[new-target]:9200/] `

* **`--config file.yml`** Override default configurations

  Allows different environments to provide different shard/replica counts, index names, etc.  
  `config.yml` is a subset of `default_configuration.py`.  Whatever is specified will override the defaults.

* **`--es-url-override`** Replace all `es_url` settings with a new host

  Allows the bulk replacement of `es_url` settings across all indexes so that during tests the es host can be 
re-targeted without requiring a full override file

* **`--no-confirm`** Skips confirmation for use in automation.
  
  By default the script will prompt before executing because this is a destructive script -- **all existing data will be dropped**
